local StacKings = _G.StacKings

local L = LibStub("AceLocale-3.0"):GetLocale("StacKings")
local LSM = LibStub("LibSharedMedia-3.0")

local function getBarChoices()
	local result = {}
	for k,v in pairs(StacKings.db.profile.bars) do
		result[tostring(k)] = v.name
	end
	return result
end

local function setOptionByName(bar, name, value)
	local settings = StacKings.db.profile.bars[bar]
	settings[name] = value
	StacKings:ApplyFrameSettings(bar, settings)
end

local function setOption(info, value)
	local bar = tonumber(info[2])
	local name = info[#info]
	setOptionByName(bar, name, value)
end

local function getOptionByName(bar, name)
	return StacKings.db.profile.bars[bar][name]
end

local function getOption(info)
	local bar = tonumber(info[2])
	local name = info[#info]
	return getOptionByName(bar, name)
end

local function getColor(info)
	local r, g, b = unpack(getOption(info))
	return r, g, b
end

local function setColor(info, r, g, b)
	local color = getOption(info)
	local bar = tonumber(info[2])
	color[1] = r
	color[2] = g
	color[3] = b
	setOption(info, color)
end

local unitChoices = {
	["player"] = PLAYER,
	["pet"] = PET,
	["target"] = TARGET,
	["focus"] = FOCUS,
}

local specChoices = {
	[0] = L["Both"],
	[1] = PRIMARY,
	[2] = SECONDARY,
}

local typeChoices = {
	["HELPFUL"] = L["Buff"],
	["HARMFUL"] = L["Debuff"],
}

local function getStatusBarChoices()
	local result = {}
	for k,v in pairs(LSM:List('statusbar')) do
		result[v] = v
	end
	return result
end

local function getFontChoices()
	local result = {}
	for k,v in pairs(LSM:List('font')) do
		result[v] = v
	end
	return result
end

local function resetPosition(info)
	local bar = tonumber(info[2])
	local point = getOptionByName(bar, "point")
	point[1] = "CENTER"
	point[2] = nil
	point[3] = "CENTER"
	point[4] = 0
	point[5] = 0
	setOptionByName(bar, "point", point)
end

local options = {
	name = L["StacKings"],
	desc = L["StacKings Buff stacks bars"],
	type = 'group',
	icon = [[Interface\GROUPFRAME\UI-Group-LeaderIcon]],
	handler = StacKings,
	set = setOption,
	get = getOption,
	args = {
		new = {
			name = L["New Bar"],
			desc = L["Create a new bar"],
			type = 'execute',
			func = "CreateBar",
			order = 2,
		},
		bars = {
			type = 'group',
			name = L["Bars"],
			desc = L["Configure Bars"],
			args = {},
		},
	},
}

local barOptionArgs = {
	enabled = {
		name = L["Enabled"],
		desc = L["Enable/Disable this bar"],
		type = 'toggle',
		order = 1,
	},
	removeBar = {
		name = L["Remove Bar"],
		desc = L["Remove a bar"],
		type = 'execute',
		confirm = true,
		confirmText = L["Are you sure?"],
		disabled = function(info)
				local index = tonumber(info[2])
				return not index or index < 0
			end,
		func = function(info)
				local bar = tonumber(info[2])
				StacKings:RemoveBar(bar)
			end,
		order = 3,
	},
	name = {
		name = L["Name"],
		desc = L["Name of the bar"],
		type = 'input',
		order = 2,
		set = function(info, value)
				setOption(info, value)
				StacKings:UpdateBarOptions()
			end,
	},
	size = {
		type = 'group',
		inline = true,
		name = L["Size"],
		desc = L["Size"],
		order = 10,
		args = {
			locked = {
				name = L["Lock"],
				desc = L["Lock the position"],
				type = 'toggle',
				order = 1,
			},
			resetPosition = {
				type = 'execute',
				name = L["Reset Position"],
				desc = L["Reset the position back to the center of the screen"],
				func = resetPosition,
				order = 2,
			},
			height = {
				name = L["Height"],
				desc = L["Height of the bar"],
				type = 'range',
				min = 0,
				max = 100,
				step = 1,
				order = 11,
			},
			width = {
				name = L["Width"],
				desc = L["Width of the bar"],
				type = 'range',
				min = 0,
				max = 1024,
				step = 10,
				order = 12,
			},
		},
	},
	behaviour = {
		type = 'group',
		inline = true,
		name = L["Behaviour"],
		desc = L["Behaviour"],
		order = 11,
		args = {
			unit = {
				type = 'select',
				name = L["Unit"],
				desc = L["Unit to watch"],
				values = unitChoices,
				order = 1,
			},
			spell = {
				type = 'input',
				name = L["Spell"],
				desc = L["Name of the spell to watch"],
				order = 2,
			},
			auratype = {
				type = 'select',
				name = L["Type"],
				desc = L["Type of the Spell"],
				order = 3,
				values = typeChoices,
			},
			max = {
				type = 'input',
				name = L["Max amount"],
				desc = L["Maximum amount of stacks"],
				order = 4,
				set = function(info, value)
						setOption(info, tonumber(value))
					end,
				get = function(info)
						local result = getOption(info)
						return tostring(result)
					end,
			},
			spec = {
				type = 'select',
				name = L["Talentspec"],
				desc = L["Limit the visbility to one spec"],
				values = specChoices,
				order = 5,
			},
		},
	},
	barstyle = {
		type = 'group',
		inline = true,
		name = L["Barstyle"],
		desc = L["Barstyle"],
		order = 12,
		args = {
			color = {
				type = 'color',
				name = L["Color"],
				desc = L["The color of the bar"],
				get = getColor,
				set = setColor,
				order = 1,
			},
			bgcolor = {
				type = 'color',
				name = L["Background"],
				desc = L["The backgroundcolor of the bar"],
				get = getColor,
				set = setColor,
				order = 2,
			},
			bgalpha = {
				type = 'range',
				name = L["Background alpha"],
				desc = L["Background alpha"],
				min = 0,
				max = 1,
				step = 0.01,
				bigStep = 0.1,
				order = 3,
			},
			bartexture = {
				type = 'select',
				control = "LSM30_Statusbar",
				values = getStatusBarChoices,
				name = L["Texture"],
				desc = L["Bartexture for the bar"],
				order = 4,
			},
		},
	},
	text = {
		type = 'group',
		inline = true,
		name = L["Text"],
		desc = L["Text displaying the remaining time"],
		order = 13,
		args = {
			showTime = {
				type = 'toggle',
				name = L["Show time"],
				desc = L["Show remaining time of the buff in the bar"],
				order = 1,
			},
			fontcolor = {
				type = 'color',
				name = L["Color"],
				desc = L["The color of the text"],
				get = getColor,
				set = setColor,
				order = 2,
			},
			fontface = {
				type = 'select',
				control = "LSM30_Font",
				values = getFontChoices,
				name = L["Font"],
				desc = L["Font of the text"],
				order = 3,
			},
			fontsize = {
				type = 'range',
				name = L["Fontsize"],
				desc = L["Fontsize of the text"],
				min = 1,
				max = 36,
				step = 1,
				order = 4,
			},
			showthreshold = {
				type = 'range',
				name = L["Time threshold"],
				desc = L["Maximum number of seconds to show the time"],
				min = 1,
				max = 1000,
				step = 1,
				bigStep = 10,
				order = 5,
			},
			flash = {
				type = 'toggle',
				name = L["Flash when expiring"],
				desc = L["Flash when buff is expiring"],
				order = 6,
			},
		},
	},
	icon = {
		type = 'group',
		inline = true,
		name = L["Icon"],
		desc = L["Settings for the icon attached to the bar"],
		order = 15,
		args = {
			iconenable = {
				type = 'toggle',
				name = L["Enable"],
				desc = L["Enable the icon"],
				order = 1,
			},
			icontexture = {
				type = 'input',
				name = L["Texture"],
				desc = L["Texture to use for the icon"],
				usage = L["Spell-ID or Spellname or Texture-path"],
				order = 2,
			},
		},
	},
}

local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

function StacKings:UpdateBarOptions()
	local barArgs = options.args.bars.args
	wipe(barArgs)
	for k,v in pairs(self.db.profile.bars) do
		local barOpt = {
			type = 'group',
			name = v.name,
			desc = v.name,
			args = barOptionArgs,
		}
		barArgs[tostring(k)] = barOpt
	end
end

function StacKings:InitOptions()
	self:UpdateBarOptions()
	AceConfig:RegisterOptionsTable("StacKings", options)
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	self:RegisterChatCommand("stack", "ShowConfig")
end

function StacKings:ShowConfig()
	AceConfigDialog:Open("StacKings")
end
